// -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-

// Copyright (c) 2001-2003 International Computer Science Institute
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software")
// to deal in the Software without restriction, subject to the conditions
// listed in the XORP LICENSE file. These conditions include: you must
// preserve this copyright notice, and you cannot mention the copyright
// holders in advertising related to the Software without their permission.
// The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
// notice is a summary of the XORP LICENSE file; the license in that file is
// legally binding.

#ident "$XORP: xorp/bgp/test_main.cc,v 1.7 2003/09/27 01:21:09 atanu Exp $"

#include <stdio.h>
#include "bgp_module.h"
#include "config.h"
#include "libxorp/xlog.h"
#include "test_next_hop_resolver.hh"

bool test_ribin(TestInfo& info);
bool test_deletion(TestInfo& info);
bool test_filter(TestInfo& info);
bool test_cache(TestInfo& info);
bool test_nhlookup(TestInfo& info);
bool test_decision(TestInfo& info);
bool test_fanout(TestInfo& info);
bool test_dump(TestInfo& info);
bool test_ribout(TestInfo& info);

int
main(int argc, char** argv) 
{
    XorpUnexpectedHandler x(xorp_unexpected_handler);

    TestMain t(argc, argv);

    string test_name =
	t.get_optional_args("-t", "--test", "run only the specified test");
    t.complete_args_parsing();

    try {
	/*
	** For next hop resolver tests.
	*/
	IPv4 nh4("128.16.64.1");
	IPv4 rnh4("1.1.1.1");
	IPv4Net nlri4("22.0.0.0/8");
	IPv6 nh6("::128.16.64.1");
	IPv6 rnh6("::1.1.1.1");
	IPv6Net nlri6("::22.0.0.0/8");
	const int iter = 1000;

	struct test {
	    string test_name;
	    XorpCallback1<bool, TestInfo&>::RefPtr cb;
	} tests[] = {
	    {"RibIn", callback(test_ribin)},
	    {"Deletion", callback(test_deletion)},
	    {"Filter", callback(test_filter)},
	    {"Cache", callback(test_cache)},
	    {"NhLookup", callback(test_nhlookup)},
	    {"Decision", callback(test_decision)},
	    {"Fanout", callback(test_fanout)},
	    {"Dump", callback(test_dump)},
	    {"Ribout", callback(test_ribout)},

	    {"nhr.test1", callback(nhr_test1<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test1.ipv6", callback(nhr_test1<IPv6>, nh6, rnh6, nlri6)},

	    {"nhr.test2", callback(nhr_test2<IPv4>, nh4, rnh4, nlri4, iter)},
	    {"nhr.test2.ipv6", callback(nhr_test2<IPv6>, nh6, rnh6, nlri6,
					iter)},

	    {"nhr.test3", callback(nhr_test3<IPv4>, nh4, rnh4, nlri4, iter)},
	    {"nhr.test3.ipv6", callback(nhr_test3<IPv6>, nh6, rnh6, nlri6,
					iter)},

	    {"nhr.test4", callback(nhr_test4<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test4.ipv6", callback(nhr_test4<IPv6>, nh6, rnh6, nlri6)},

	    {"nhr.test5", callback(nhr_test5<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test5.ipv6", callback(nhr_test5<IPv6>, nh6, rnh6, nlri6)},

	    {"nhr.test6", callback(nhr_test6<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test6.ipv6", callback(nhr_test6<IPv6>, nh6, rnh6, nlri6)},

	    {"nhr.test7", callback(nhr_test7<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test7.ipv6", callback(nhr_test7<IPv6>, nh6, rnh6, nlri6)},

	    {"nhr.test8", callback(nhr_test8<IPv4>, nh4, rnh4, nlri4)},
	    {"nhr.test8.ipv6", callback(nhr_test8<IPv6>, nh6, rnh6, nlri6)},
	};

	if("" == test_name) {
	    for(unsigned int i = 0; i < sizeof(tests) / sizeof(struct test); 
		i++)
		t.run(tests[i].test_name, tests[i].cb);
	} else {
	    for(unsigned int i = 0; i < sizeof(tests) / sizeof(struct test); 
		i++)
		if(test_name == tests[i].test_name) {
		    t.run(tests[i].test_name, tests[i].cb);
		    return t.exit();
		}
	    t.failed("No test with name " + test_name + " found\n");
	}
    } catch(...) {
	xorp_catch_standard_exceptions();
    }

    return t.exit();
}
